#
# LightCGIHTTPServer.py - Tiny HTTP server supporting threading CGI.
# Copyright (C) 2005  shinGETsu Project.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: LightCGIHTTPServer.py,v 1.9 2005/04/24 13:12:27 fuktommy Exp $
#

import os
import re
import sys
import urllib
import BaseHTTPServer
import CGIHTTPServer
import SocketServer

import admin_cgi, client_cgi, server_cgi
import gateway_cgi, list_cgi, note_cgi, thread_cgi

cgimodule = {"admin.cgi": admin_cgi,
             "client.cgi": client_cgi,
             "server.cgi": server_cgi,
             "gateway.cgi": gateway_cgi,
             "list.cgi": list_cgi,
             "note.cgi": note_cgi,
             "thread.cgi": thread_cgi}

__version__ = "$Revision: 1.9 $"

class HTTPRequestHandler(CGIHTTPServer.CGIHTTPRequestHandler):

    """Tiny HTTP Server.

    Exec CGI if script name ends with .cgi.
    The CGI script runs in same process.
    """
    root_index = "/"

    # Make rfile buffered -- we don't use subprocess.
    rbufsize = -1

    def is_cgi(self):
        """Test request URI is *.cgi."""
        found = re.search(r"^(.*?)/([^/]+\.cgi.*)", self.path)
        if found:
            self.cgi_info = found.groups()
            return True
        else:
            return False

    def common(self):
        if self.path == "/":
            self.path = self.root_index
        os.environ["HTTP_ACCEPT_LANGUAGE"] = \
            self.headers.get("Accept-Language", "")
        os.environ["HTTP_HOST"] = self.headers.getheader('host')
        os.environ["HTTP_REFERER"] = self.headers.get("Referer", "")

    def do_POST(self):
        self.common()
        return CGIHTTPServer.CGIHTTPRequestHandler.do_POST(self)

    def send_head(self):
        self.common()
        return CGIHTTPServer.CGIHTTPRequestHandler.send_head(self)

    def log_request(self, code='-', size='-'):
        buf = [self.requestline]
        if hasattr(self, "headers"):
            buf.extend((self.headers.get("Referer", ""),
                        self.headers.get("User-Agent", "")))
        self.log_message("%s", "<>".join(buf))

    def log_message(self, format, *args):
        sys.stderr.write("%s<>%s<>%s\n" %
                         (self.address_string(),
                          self.log_date_time_string(),
                          format%args))

    def common(self):
        if self.path == "/":
            self.path = self.root_index

    def run_cgi(self):
        """Execute a CGI script in this process.

        This code is from CGIHTTPServer.py module.
        """
        dir, rest = self.cgi_info
        i = rest.rfind('?')
        if i >= 0:
            rest, query = rest[:i], rest[i+1:]
        else:
            query = ''
        i = rest.find('/')
        if i >= 0:
            script, rest = rest[:i], rest[i:]
        else:
            script, rest = rest, ''
        scriptname = dir + '/' + script
        scriptfile = self.translate_path(scriptname)

        # Reference: http://hoohoo.ncsa.uiuc.edu/cgi/env.html
        # XXX Much of the following could be prepared ahead of time!
        env = {}
        env['SERVER_SOFTWARE'] = self.version_string()
        env['SERVER_NAME'] = self.server.server_name
        env['GATEWAY_INTERFACE'] = 'CGI/1.1'
        env['SERVER_PROTOCOL'] = self.protocol_version
        env['SERVER_PORT'] = str(self.server.server_port)
        env['REQUEST_METHOD'] = self.command
        uqrest = urllib.unquote(rest)
        env['PATH_INFO'] = uqrest
        env['PATH_TRANSLATED'] = self.translate_path(uqrest)
        env['SCRIPT_NAME'] = scriptname
        if query:
            env['QUERY_STRING'] = query
        host = self.address_string()
        if host != self.client_address[0]:
            env['REMOTE_HOST'] = host
        env['REMOTE_ADDR'] = self.client_address[0]
        # XXX AUTH_TYPE
        # XXX REMOTE_USER
        # XXX REMOTE_IDENT
        if self.headers.typeheader is None:
            env['CONTENT_TYPE'] = self.headers.type
        else:
            env['CONTENT_TYPE'] = self.headers.typeheader
        length = self.headers.getheader('content-length')
        if length:
            env['CONTENT_LENGTH'] = length
        accept = []
        for line in self.headers.getallmatchingheaders('accept'):
            if line[:1] in "\t\n\r ":
                accept.append(line.strip())
            else:
                accept = accept + line[7:].split(',')
        env['HTTP_ACCEPT'] = ','.join(accept)
        ua = self.headers.getheader('user-agent')
        if ua:
            env['HTTP_USER_AGENT'] = ua
        co = filter(None, self.headers.getheaders('cookie'))
        if co:
            env['HTTP_COOKIE'] = ', '.join(co)
        # XXX Other HTTP_* headers
        # Since we're setting the env in the parent, provide empty
        # values to override previously set values
        for k in ('QUERY_STRING', 'REMOTE_HOST', 'CONTENT_LENGTH',
                  'HTTP_USER_AGENT', 'HTTP_COOKIE'):
            env.setdefault(k, "")

        # HTTP_* headers require by SAKU
        env["HTTP_ACCEPT_LANGUAGE"] = \
            self.headers.get("Accept-Language", "")
        env["HTTP_HOST"] = self.headers.getheader('host')
        env["HTTP_REFERER"] = self.headers.get("Referer", "")

        # import CGI module
        try:
            cgiclass = cgimodule[script].CGI
        except KeyError:
            self.send_error(404, "No such CGI script (%s)" % `scriptname`)
            return
        self.send_response(200, "Script output follows")

        decoded_query = query.replace('+', ' ')

        # execute script in this process
        try:
            cgiobj = cgiclass(stdin=self.rfile,
                              stdout=self.wfile,
                              stderr=sys.stderr,
                              environ=env)
            cgiobj.start()
        except SystemExit, sts:
            self.log_error("CGI script exit status %s", str(sts))


class HTTPServer(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer):
    pass


def test(HandlerClass=None, ServerClass=None, port=8000):
    import SimpleHTTPServer
    if HandlerClass is None:
        HandlerClass = HTTPRequestHandler
    if ServerClass is None:
        ServerClass = HTTPServer
    server_address = ('', port)
    httpd = ServerClass(server_address, HandlerClass)
    sa = httpd.socket.getsockname()
    print "Serving HTTP on", sa[0], "port", sa[1], "..."
    httpd.serve_forever()

if __name__ == '__main__':
    test()
