#
# config.py - Configuration.
# Copyright (C) 2005  shinGETsu Project.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: config.py,v 1.35 2005/04/17 07:32:11 fuktommy Exp $
#

import os.path
import sys
import ConfigParser

__version__ = "$Revision: 1.35 $"

# External config files.
config_files = ["file/saku.ini",
                "/usr/local/etc/saku.ini",
                "/etc/saku.ini",
                os.path.expanduser("~/.saku/saku.ini")]

extconf = ConfigParser.SafeConfigParser()
extconf.read(config_files)

def get_value(parser, key, default, vtype=""):
    """Get config value or default value."""
    try:
        if vtype == "int":
            return parser.getint("SAKU", key)
        elif vtype == "path":
            return os.path.expanduser(parser.get("SAKU", key))
        else:
            return parser.get("SAKU", key)
    except (ConfigParser.NoOptionError,
            ConfigParser.NoSectionError):
                return default

port = get_value(extconf, "port", 8000, "int")
tklog = get_value(extconf, "tklog", 100, "int")
admin = get_value(extconf, "admin", r"^127")
docroot = get_value(extconf, "docroot", "./www", "path")
log_dir = get_value(extconf, "log_dir", "./log", "path")
run_dir = get_value(extconf, "run_dir", "../run", "path")
file_dir = get_value(extconf, "file_dir", "../file", "path")
cache_dir = get_value(extconf, "cache_dir", "../cache", "path")

client_cycle = 5*60         # Seconds; Access client.cgi
ping_cycle = 5*60           # Seconds; Check nodes
sync_cycle = 5*60*60        # Seconds; Check cache
init_cycle = 20*60          # Seconds; Check initial node
sync_range = 10*24*60*60    # Seconds
update_range = 24*60*60     # Seconds
rss_range = 3*24*60*60      # Seconds
wait_update = 10            # Seconds
time_error = 60             # Seconds
save_history = 20*24*60*60  # Seconds
save_removed = 30*24*60*60  # Seconds
search_timeout = 10*60      # Seconds

timeout = 10                # Seconds; Timeout for TCP
get_timeout = 2*60          # Seconds; Timeout for /get
client_timeout = 30*60      # Seconds; client_timeout < sync_cycle
retry = 5                   # Times; Common setting
nodes = 5                   # Nodes keeping in node list
search_depth = 30           # Search node size
file_limit = 10             # MegaBytes; File size limit

broadcast = "../tool/broadcast.py"  # Broadcast script path

rss = "1"                           # 1/2; RSS version
google = "http://www.google.co.jp/search"
robot = r"bot|Yahoo|archiver";      # regexp
language = "en"                     # Language code (see RFC3066)

dnsname = ""                            # Server name
menu = "menu"                           # File name for /list.cgi
query_separator = "/"                   # Charactor for foo.cgi[/?]bar
root_path = "/"                         # path of URI for root

motd = file_dir + "/motd.txt"
node = run_dir + "/node.txt"
search = run_dir + "/search.txt"
update = run_dir + "/update.txt"
client_log = run_dir + "/client.txt"
lock = run_dir + "/lock.txt"
search_lock = run_dir + "/touch.txt"

server = root_path + "server.cgi"
client = root_path + "client.cgi"
gateway = root_path + "gateway.cgi"
list_cgi = root_path + "list.cgi"
thread_cgi = root_path + "thread.cgi"
note_cgi = root_path + "note.cgi"
admin_cgi = root_path + "admin.cgi"
css = root_path + "default.css"

types = ("list", "thread", "note")      # Application types
application = {"list": list_cgi,
               "thread": thread_cgi,
               "note": note_cgi}

version = "shinGETsu/0.5 (SAKU/0.3.0)"

init_node = ["shingetsu.xrea.jp:80/node.cgi",
             "fuktommy.ddo.jp:8000/server.cgi",
             "feeling.zive.net:8000/server.cgi",
             "pushare.zive.net:8000/server.cgi"]

ignore_search_node = ["shingetsu.xrea.jp:80/node.cgi"]

flags = []          # It is set by script
